* Output options: all can be turned on/off by adding/ removing the NO;
  OPTIONS NOnumber NOdate NOcenter FormDlim=' ' PageSize=MAX LineSize=MAX NOfmterr;
* Remove default title, turn on new HTML graphics;
  TITLE; ODS GRAPHICS ON; ODS LISTING;  

********************************************************************************;
*******    FIRST DEFINE "FILESAVE" PLACEHOLDER FOR FOLDER LOCATION       *******;
*******          BEGIN EXAMPLE 3 DATA IMPORT AND MANIPULATION            *******;  
********************************************************************************;

* Define global variable for file location to be replaced in code below;
* \\Client\ precedes path in Virtual Desktop outside H drive;
%LET filesave= C:\Dropbox\22SP_PSQF6243\PSQF6243_Example3;

* IMPORT GSS_Example.xlsx data using filesave reference and exact file name;
* from the Excel workbook in DATAFILE= location from SHEET= ;
* New SAS file is in "work" library place with name "Example3";
* "GETNAMES" reads in the first row as variable names;
* DBMS=XLSX (can also use EXCEL or XLS for .xls files);
PROC IMPORT DATAFILE="&filesave.\GSS_Example.xlsx" 
            OUT=work.Example3 DBMS=XLSX REPLACE; 
     SHEET="GSS_Example"; 
     GETNAMES=YES; 
RUN;

* All data transformations must happen inside a DATA+SET combo to know where to use them;
* Here is how to make a new variable: new = old;
DATA work.Example3; SET work.Example3;
* Label variables and apply value formats for variables used below;
* LABEL name=      "name: Descriptive Variable Label";
  LABEL workclass= "workclass: 1=Lower, 2=Middle, 3=Upper"
        age=       "age: Years of Age"
        educ=      "educ: Years of Education"
        happy=     "happy: 5-Category Happy Rating"
        income=    "income: Annual Income in 1000s";
* Select cases complete on all variables to be used;
  WHERE NMISS(income,workclass,age,educ,happy)=0;
RUN;
* Now dataset work.Example3 is ready to use; 

* Open output directory to save results to;
ODS RTF FILE="&filesave.\Example3_SAS_Output.rtf" 
        STYLE=HTMLBlue STARTPAGE=NO BODYTITLE;


******************************************************************************;
*******             BEGIN EXAMPLE 3 DATA DESCRIPTION                   *******;
******************************************************************************;

TITLE "SAS Descriptive Statistics for Quantitative and Ordinal Variables";
PROC MEANS NDEC=3 DATA=work.Example3; 
     VAR income age educ happy;
RUN; TITLE;

TITLE "SAS Descriptive Statistics for Categorical Variables";
PROC FREQ DATA=work.Example3; 
     TABLE workclass happy; 
RUN; TITLE;


******************************************************************************;
*******       BEGIN PREDICTION OF INCOME BY 3-CATEGORY WORKCLASS       *******;
******************************************************************************;

* SAS code to create 2 new indicator-dummy-coded binary predictors;
* DATA + SET means "save as itself" after adding new predictors;
DATA work.Example3; SET work.Example3;
  LvM=.; LvU=.; * Make 2 new empty variables;
  IF workclass=1 THEN DO; LvM=0; LvU=0; END; * Replace each for lower;
  IF workclass=2 THEN DO; LvM=1; LvU=0; END; * Replace each for middle;
  IF workclass=3 THEN DO; LvM=0; LvU=1; END; * Replace each for upper; 
  LABEL LvM="LvM: Low=0 vs Mid=1 Class"
        LvU="LvU: Low=0 vs Upp=1 Class";
RUN;

TITLE "SAS GLM Predicting Income from 2 New Variables for workclass";
PROC GLM DATA=work.Example3 NAMELEN=100; 
  MODEL income = LvM LvU / SOLUTION ALPHA=.05 CLPARM SS3;
* Ask for predicted income per group and group differences;
  ESTIMATE "Pred Income: Low"  intercept 1 LvM  0 LvU 0;
  ESTIMATE "Pred Income: Mid"  intercept 1 LvM  1 LvU 0;
  ESTIMATE "Pred Income: Upp"  intercept 1 LvM  0 LvU 1;
  ESTIMATE "Low vs Mid Diff"               LvM  1 LvU 0;
  ESTIMATE "Low vs Upp Diff"               LvM  0 LvU 1;
  ESTIMATE "Mid vs Upp Diff"               LvM -1 LvU 1;
* Save requested estimates as SAS dataset to do math on them;
  ODS OUTPUT Estimates=work.ClassEstimates;
RUN; QUIT; TITLE;

* SAS code to compute effect sizes from stored ESTIMATE results;
DATA work.ClassEstimates; SET work.ClassEstimates;
* Cohen d is partial standardized mean difference; 
  PartialD=(2*tValue)/SQRT(731); * SQRT(number) = DF denominator;
* PartialR is partial correlation;
  PartialR = tvalue/(SQRT(tvalue**2 +731)); * +number = DF denominator;
RUN;
* Print estimates table with effect sizes added;
TITLE "CohenD and PartialR Effect Sizes for 3-Category workclass";
PROC PRINT NOOBS DATA=work.ClassEstimates; 
     VAR Parameter--PartialR; * Print all contiguous columns; 
RUN; TITLE;


******************************************************************************;
*******         BEGIN PREDICTION OF INCOME BY QUADRATIC AGE            *******;
******************************************************************************;

* SAS code to create 1 new age variable centered at 18 (minimum in sample);
DATA work.Example3; SET work.Example3;
  age18=age-18; 
  LABEL age18= "age18: Age (0=18 years)";
RUN;

TITLE "SAS GLM Predicting Income from Linear Centered Age (0=18)";
PROC GLM DATA=work.Example3 NAMELEN=100; 
  MODEL income = age18 / SOLUTION ALPHA=.05 CLPARM SS3;
* Ask for predicted income for example ages;
  ESTIMATE "Pred Income: Age 30 (age18=12)"  intercept 1 age18 12;
  ESTIMATE "Pred Income: Age 50 (age18=32)"  intercept 1 age18 32;
  ESTIMATE "Pred Income: Age 70 (age18=52)"  intercept 1 age18 52;
RUN; QUIT; TITLE; 

TITLE "SAS GLM Predicting Income from Linear+Quadratic Centered Age";
PROC GLM DATA=work.Example3 NAMELEN=100;
* Asterisk creates multiplied predictor variable;
  MODEL income = age18 age18*age18 / SOLUTION ALPHA=.05 CLPARM SS3;
* Ask for predicted income for example ages;
  ESTIMATE "Pred Income: Age 30 (age18=12)"  intercept 1 age18 12 age18*age18  144;
  ESTIMATE "Pred Income: Age 50 (age18=32)"  intercept 1 age18 32 age18*age18 1024;
  ESTIMATE "Pred Income: Age 70 (age18=52)"  intercept 1 age18 52 age18*age18 2704;
* Linear age slope changes by 2*quadratic coefficient per year, so multiply age*2;
  ESTIMATE "Pred Linear Age Slope: Age 30 (age18=12)"    age18 1  age18*age18  24;
  ESTIMATE "Pred Linear Age Slope: Age 50 (age18=32)"    age18 1  age18*age18  64;
  ESTIMATE "Pred Linear Age Slope: Age 70 (age18=52)"    age18 1  age18*age18 104;
* Save predicted income and SE to new dataset to make pictures;
  OUTPUT OUT=work.PredIncomebyAge PREDICTED=YhatAge STDP=SEyhatAge;
RUN; QUIT; TITLE; 
* Not getting partial R effect sizes because linear+quadratic slopes work together;

******************************************************************************;
*******  BEGIN PREDICTION OF INCOME BY PIECEWISE SLOPES FOR EDUCATION  *******;
******************************************************************************;

* SAS code to create 3 new predictor variables for sections of education;
DATA work.Example3; SET work.Example3;
  lessHS=.; gradHS=.; overHS=.; * Make 3 new empty variables;
* Replace each for educ less than 12;
  IF educ LT 12 THEN DO; lessHS=educ-11; gradHS=0; overHS=0;       END;
* Replace each for educ greater or equal to 12;
  IF educ GE 12 THEN DO; lessHS=0;       gradHS=1; overHS=educ-12; END;
  LABEL lessHS= "lessHS: Slope for Years Ed Less Than High School"
        gradHS= "gradHS: Bump for Graduating High School"
        overHS= "overHS: Slope for Years Ed After High School";
RUN;

TITLE "SAS GLM Predicting Income from 3 Piecewise Linear Slopes for Education";
PROC GLM DATA=work.Example3 NAMELEN=100;
  MODEL income = lessHS gradHS overHS / SOLUTION ALPHA=.05 CLPARM SS3;
* Example of how to compare slopes;
  ESTIMATE "Diff in ed slope:  2-11 vs 11-12" lessHS -1 gradHS 1;
  ESTIMATE "Diff in ed slope: 11-12 vs 12-20" gradHS -1 overHS 1;
* Save predicted income and SE to new dataset to make pictures;
  OUTPUT OUT=work.PredIncomebyEduc PREDICTED=YhatEduc STDP=SEyhatEduc;
* Save fixed effect estimates and requested estimates as SAS datasets to do math on them;
  ODS OUTPUT ParameterEstimates=work.EducSolution Estimates=work.EducEstimates;
RUN; QUIT; TITLE;

* SAS code to compute effect sizes from stored fixed effect results;
DATA work.EducEffectSizes; LENGTH Parameter $50;
  SET work.EducSolution work.EducEstimates; * Combine tables;
* PartialR is partial correlation (using +DFden);
  PartialR = tvalue/(SQRT(tvalue**2 +730)); * +number = DF denominator;
RUN;
* Print estimates table with effect sizes added;
TITLE "PartialR Effect Sizes for Piecewise Slopes for Education";
PROC PRINT NOOBS DATA=work.EducEffectSizes; 
     VAR Parameter--PartialR; * Print all contiguous columns;
RUN; TITLE;

* SAS alternative method to compute partial correlations for fixed slopes;
TITLE "SAS Partial Correlation of income with lessHS";
PROC CORR NOSIMPLE DATA=work.Example3; VAR income lessHS; PARTIAL gradHS overHS; RUN;
TITLE "SAS Partial Correlation of income with gradHS";
PROC CORR NOSIMPLE DATA=work.Example3; VAR income gradHS; PARTIAL lessHS overHS; RUN;
TITLE "SAS Partial Correlation of income with overHS";
PROC CORR NOSIMPLE DATA=work.Example3; VAR income overHS; PARTIAL lessHS gradHS; RUN; 
TITLE;


******************************************************************************;
*******   BEGIN PREDICTION OF INCOME BY ORDINAL 5-CATEGORY HAPPINESS   *******;
******************************************************************************;

* SAS code to create 1 new happy variable centered at lowest value;
DATA work.Example3; SET work.Example3;
  happy1=happy-1;
  LABEL happy1= "happy1: Happy Category (0=1)";
RUN;

TITLE "SAS GLM Predicting Income from Linear Centered Happy (0=1)";
PROC GLM DATA=work.Example3 NAMELEN=100;
  MODEL income = happy1 / SOLUTION ALPHA=.05 CLPARM SS3;
* Save predicted income and SE to new dataset to make pictures;
  OUTPUT OUT=work.PredIncomebyHappy1 PREDICTED=Yhat1Happy STDP=SEyhat1Happy;
RUN; QUIT; TITLE;

* SAS code to create 4 new sequential-dummy-coded binary predictors for happy;
DATA work.Example3; SET work.Example3;
  h1v2=.; h2v3=.; h3v4=.; h4v5=.; * Make 4 new empty variables;
  IF happy=1 THEN DO; h1v2=0; h2v3=0; h3v4=0; h4v5=0; END; * Replace each for happy=1;
  IF happy=2 THEN DO; h1v2=1; h2v3=0; h3v4=0; h4v5=0; END; * Replace each for happy=2;
  IF happy=3 THEN DO; h1v2=1; h2v3=1; h3v4=0; h4v5=0; END; * Replace each for happy=3;
  IF happy=4 THEN DO; h1v2=1; h2v3=1; h3v4=1; h4v5=0; END; * Replace each for happy=4;
  IF happy=5 THEN DO; h1v2=1; h2v3=1; h3v4=1; h4v5=1; END; * Replace each for happy=5;
  LABEL h1v2="Slope from Happy 1 to 2"
        h2v3="Slope from Happy 2 to 3"
        h3v4="Slope from Happy 3 to 4"
        h4v5="Slope from Happy 4 to 5";
RUN;

TITLE "SAS GLM Predicting Income from Sequential Slopes for Happy";
PROC GLM DATA=work.Example3 NAMELEN=100;
  MODEL income = h1v2 h2v3 h3v4 h4v5 / SOLUTION ALPHA=.05 CLPARM SS3;
* Example of how to compare slopes;
  ESTIMATE "Diff in Slope 1-2 vs 2-3" h1v2 -1 h2v3 1;
  ESTIMATE "Diff in Slope 2-3 vs 3-4" h2v3 -1 h3v4 1;
  ESTIMATE "Diff in Slope 3-4 vs 4-5" h3v4 -1 h4v5 1;
* Save fixed effect estimates and requested estimates as SAS datasets to do math on them;
  ODS OUTPUT ParameterEstimates=work.HappySolution Estimates=HappyEstimates;
RUN; QUIT; TITLE;

* SAS code to compute effect sizes from stored fixed effect results;
DATA work.HappyEffectSizes; LENGTH Parameter $50;
  SET work.HappySolution work.HappyEstimates; * Combine tables;
* PartialR is partial correlation using +DFden);
  PartialR = tvalue/(SQRT(tvalue**2+729));
RUN;
* Print estimates table with effect sizes added;
TITLE "PartialR Effect Sizes for Sequential Slopes for Happy";
PROC PRINT NOOBS DATA=work.HappyEffectSizes; 
     VAR Parameter--PartialR; * Print all contiguous columns;
RUN; TITLE;

* SAS alternative method to compute partial correlations for fixed slopes;
TITLE "SAS Partial Correlation of income with h1v2";
PROC CORR NOSIMPLE DATA=work.Example3; VAR income h1v2; PARTIAL h2v3 h3v4 h4v5; RUN;
TITLE "SAS Partial Correlation of income with h2v3";
PROC CORR NOSIMPLE DATA=work.Example3; VAR income h2v3; PARTIAL h1v2 h3v4 h4v5; RUN;
TITLE "SAS Partial Correlation of income with h3v4";
PROC CORR NOSIMPLE DATA=work.Example3; VAR income h3v4; PARTIAL h1v2 h2v3 h4v5; RUN;
TITLE "SAS Partial Correlation of income with h4v5";
PROC CORR NOSIMPLE DATA=work.Example3; VAR income h4v5; PARTIAL h1v2 h2v3 h3v4; RUN;
TITLE;


***************************************************************************************;
*******     BONUS CODE TO MAKE PICTURES FOR PREDICTED INCOME BY PREDICTOR       *******;
*******     UESS RESULTS SAVED FROM ABOVE TO COMPARE AGAINST ORIGINAL DATA      *******;
***************************************************************************************;

* Save descriptive statistics for personal income by unqiue predictor value;
PROC MEANS NOPRINT NDEC=3 DATA=work.Example3; 
     CLASS age educ happy; WAYS 1; * WAYS=1 means only run for each variable separately;
     VAR income;
     OUTPUT OUT=work.Means 
            MEAN(income)=meanincome STDERR(income)=SEincome;
RUN; TITLE;

* Compute -1 and +1 SE away from means;
DATA work.Means; SET work.Means;
     lowerSEincome=meanincome-SEincome;
     upperSEincome=meanincome+SEincome;
RUN;

* AGE MODELS;

TITLE "SAS Line Plot for Income Means Per Year of Age";
PROC SGPLOT DATA=work.Means; WHERE NMISS(age)=0;
     SERIES x=age y=meanincome  / LINEATTRS=(COLOR=BLACK PATTERN=SOLID);
     SERIES x=age y=lowerSEincome / LINEATTRS=(COLOR=RED PATTERN=SOLID);
     SERIES x=age y=upperSEincome / LINEATTRS=(COLOR=RED PATTERN=SOLID);
     XAXIS LABEL="Years of Age" VALUES=(18 TO 76 BY 2);
     YAXIS LABEL="1000s of Income" VALUES=(0 TO 55 BY 5);
RUN;

* Sort by age;
PROC SORT DATA=work.PredIncomebyAge; 
     BY age18; 
RUN;
* Compute means of predicted income from age for each unique value of age;
PROC MEANS NOPRINT DATA=work.PredIncomebyAge; 
     BY age18; 
     VAR YhatAge SEyhatAge;
     OUTPUT OUT=work.MeanYhatAge MEAN(YhatAge)=YhatAge MEAN(SEyhatAge)=SEyhatAge; 
RUN;
* Compute uncentered age for plot;
DATA work.MeanYhatAge; SET work.MeanYhatAge;
     age=age18+18; 
RUN;
* Merge with means data, compute -1 and +1 SE for yhat from age;
DATA work.Means; MERGE work.Means work.MeanYhatAge; BY age; 
     lowerSEage=YhatAge-SEyhatAge;
     upperSEage=YhatAge+SEyhatAge; 
RUN;
     
TITLE "SAS Line Plot for Income Means Per Year of Age";
PROC SGPLOT DATA=work.Means; WHERE NMISS(age)=0;
     SERIES x=age y=meanincome  / LINEATTRS=(COLOR=BLACK PATTERN=SOLID);
     SERIES x=age y=lowerSEincome / LINEATTRS=(COLOR=RED PATTERN=SOLID);
     SERIES x=age y=upperSEincome / LINEATTRS=(COLOR=RED PATTERN=SOLID);
     SERIES x=age y=YhatAge  / LINEATTRS=(COLOR=BLACK PATTERN=DASH);
     SERIES x=age y=lowerSEage / LINEATTRS=(COLOR=RED PATTERN=DASH);
     SERIES x=age y=upperSEage / LINEATTRS=(COLOR=RED PATTERN=DASH);
     XAXIS LABEL="Years of Age" VALUES=(18 TO 76 BY 2);
     YAXIS LABEL="1000s of Income" VALUES=(-20 TO 60 BY 10);
RUN;

* EDUC MODELS;

TITLE "SAS Line Plot for Income Means Per Year of Education";
PROC SGPLOT DATA=work.Means; WHERE NMISS(educ)=0;
     SERIES x=educ y=meanincome  / LINEATTRS=(COLOR=BLACK PATTERN=SOLID);
     SERIES x=educ y=lowerSEincome / LINEATTRS=(COLOR=RED PATTERN=SOLID);
     SERIES x=educ y=upperSEincome / LINEATTRS=(COLOR=RED PATTERN=SOLID);
     XAXIS LABEL="Years of Education" VALUES=(2 TO 20 BY 1);
     YAXIS LABEL="1000s of Income" VALUES=(0 TO 40 BY 5);
     REFLINE 11 / AXIS=X;
     REFLINE 12 / AXIS=X;
RUN;

* Sort by educ;
PROC SORT DATA=work.PredIncomebyEduc; 
     BY educ; 
RUN;
* Compute means of predicted income from educ for each unique value of educ;
PROC MEANS NOPRINT DATA=work.PredIncomebyEduc; 
     BY educ; 
     VAR YhatEduc SEyhatEduc;
     OUTPUT OUT=work.MeanYhatEduc MEAN(YhatEduc)=YhatEduc MEAN(SEyhatEduc)=SEyhatEduc;
RUN;
* Merge with means data, compute -1 and +1 SE for yhat from educ;
PROC SORT DATA=work.Means; 
     BY educ; 
RUN;
DATA work.Means; MERGE work.Means work.MeanYhatEduc; BY educ; 
     lowerSEeduc=YhatEduc-SEyhatEduc;
     upperSEeduc=YhatEduc+SEyhatEduc; 
RUN;

TITLE "SAS Predicted Income By Year of Education: Linear OverHS";
PROC SGPLOT DATA=work.Means; WHERE NMISS(educ)=0;
     SERIES x=educ y=meanincome  / LINEATTRS=(COLOR=BLACK PATTERN=SOLID);
     SERIES x=educ y=lowerSEincome / LINEATTRS=(COLOR=RED PATTERN=SOLID);
     SERIES x=educ y=upperSEincome / LINEATTRS=(COLOR=RED PATTERN=SOLID);
     SERIES x=educ y=YhatEduc  / LINEATTRS=(COLOR=BLACK PATTERN=DASH);
     SERIES x=educ y=lowerSEeduc / LINEATTRS=(COLOR=RED PATTERN=DASH);
     SERIES x=educ y=upperSEeduc / LINEATTRS=(COLOR=RED PATTERN=DASH);
     XAXIS LABEL="Years of Education" VALUES=(2 TO 20 BY 1);
     YAXIS LABEL="1000s of Income" VALUES=(0 TO 40 BY 5);
     REFLINE 11 / AXIS=X;
     REFLINE 12 / AXIS=X;
     REFLINE 15 / AXIS=X;
     REFLINE 16 / AXIS=X;
     REFLINE 17 / AXIS=X;
     REFLINE 18 / AXIS=X;
RUN;

* HAPPY MODELS;

TITLE "SAS Line Plot for Income Means Per Level of Happy";
PROC SGPLOT DATA=work.Means; WHERE NMISS(happy)=0;
     SERIES x=happy y=meanincome  / LINEATTRS=(COLOR=BLACK PATTERN=SOLID);
     XAXIS LABEL="Ordinal Happiness" VALUES=(1 TO 5 BY 1);
     YAXIS LABEL="1000s of Income" VALUES=(0 TO 30 BY 5);
RUN;

* Sort by happy;
PROC SORT DATA=work.PredIncomebyHappy1; 
     BY happy; 
RUN;
* Compute means of predicted income from age for each unique value of age;
PROC MEANS NOPRINT DATA=work.PredIncomebyHappy1; 
     BY happy; 
     VAR Yhat1Happy SEyhat1Happy;
     OUTPUT OUT=work.MeanYhat1Happy MEAN(Yhat1Happy)=Yhat1Happy MEAN(SEyhat1Happy)=SEyhat1Happy; 
RUN;
* Merge with means data, compute -1 and +1 SE for yhat from age;
PROC SORT DATA=work.Means; 
     BY happy; 
RUN;
DATA work.Means; MERGE work.Means work.MeanYhat1Happy; BY happy; 
     lowerSE1Happy=Yhat1Happy-SEyhat1Happy;
     upperSE1Happy=Yhat1Happy+SEyhat1Happy; 
RUN;
     
TITLE "SAS Line Plot for Income Means Per Level of Happy: Linear Slope";
PROC SGPLOT DATA=work.Means; WHERE NMISS(happy)=0;
     SERIES x=happy y=meanincome  / LINEATTRS=(COLOR=BLACK PATTERN=SOLID);
     SERIES x=happy y=lowerSEincome / LINEATTRS=(COLOR=RED PATTERN=SOLID);
     SERIES x=happy y=upperSEincome / LINEATTRS=(COLOR=RED PATTERN=SOLID);
     SERIES x=happy y=Yhat1Happy  / LINEATTRS=(COLOR=BLACK PATTERN=DASH);
     SERIES x=happy y=lowerSE1Happy / LINEATTRS=(COLOR=RED PATTERN=DASH);
     SERIES x=happy y=upperSE1Happy / LINEATTRS=(COLOR=RED PATTERN=DASH);
     XAXIS LABEL="Ordinal Happiness" VALUES=(1 TO 5 BY 1);
     YAXIS LABEL="1000s of Income" VALUES=(5 TO 25 BY 5);
RUN;


* Close output;
ODS RTF CLOSE;
